@@ -128,11 +128,14 @@ close_all_fds (int first, int *except)
 
 
 /* Returns an array with all currently open file descriptors.  The end
-   of the array is marked by -1.  The caller needs to release this
-   array using the *standard free* and not with xfree.  This allow the
-   use of this function right at startup even before libgcrypt has
-   been initialized.  Returns NULL on error and sets ERRNO
-   accordingly.  */
+ * of the array is marked by -1.  The caller needs to release this
+ * array using the *standard free* and not with xfree.  This allow the
+ * use of this function right at startup even before libgcrypt has
+ * been initialized.  Returns NULL on error and sets ERRNO
+ * accordingly.  Note that fstat prints a warning to DebugView for all
+ * invalid fds which is a bit annoying.  We actually do not need this
+ * function in real code (close_all_fds is a dummy anyway) but we keep
+ * it for use by t-exechelp.c.  */
 int *
 get_all_open_fds (void)
 {
@@ -306,6 +309,7 @@ create_pipe_and_estream (int filedes[2], int flags,
 {
   gpg_error_t err = 0;
   HANDLE fds[2];
+  es_syshd_t syshd;
 
   filedes[0] = filedes[1] = -1;
   err = my_error (GPG_ERR_GENERAL);
@@ -334,10 +338,17 @@ create_pipe_and_estream (int filedes[2], int flags,
 
   if (! err && r_fp)
     {
+      syshd.type = ES_SYSHD_HANDLE;
       if (!outbound)
-        *r_fp = es_fdopen (filedes[0], nonblock? "r,nonblock" : "r");
+        {
+          syshd.u.handle = fds[0];
+          *r_fp = es_sysopen (&syshd, nonblock? "r,nonblock" : "r");
+        }
       else
-        *r_fp = es_fdopen (filedes[1], nonblock? "w,nonblock" : "w");
+        {
+          syshd.u.handle = fds[1];
+          *r_fp = es_sysopen (&syshd, nonblock? "w,nonblock" : "w");
+        }
       if (!*r_fp)
         {
           err = my_error_from_syserror ();
@@ -403,9 +414,11 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
       0,         /* Returns pid.  */
       0          /* Returns tid.  */
     };
-  STARTUPINFO si;
+  STARTUPINFOW si;
   int cr_flags;
   char *cmdline;
+  wchar_t *wcmdline = NULL;
+  wchar_t *wpgmname = NULL;
   HANDLE inpipe[2]  = {INVALID_HANDLE_VALUE, INVALID_HANDLE_VALUE};
   HANDLE outpipe[2] = {INVALID_HANDLE_VALUE, INVALID_HANDLE_VALUE};
   HANDLE errpipe[2] = {INVALID_HANDLE_VALUE, INVALID_HANDLE_VALUE};
@@ -415,7 +428,7 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
   HANDLE nullhd[3] = {INVALID_HANDLE_VALUE,
                       INVALID_HANDLE_VALUE,
                       INVALID_HANDLE_VALUE};
-  int i;
+  int i, rc;
   es_syshd_t syshd;
   gpg_err_source_t errsource = default_errsource;
   int nonblock = !!(flags & GNUPG_SPAWN_NONBLOCK);
@@ -544,7 +557,7 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
   memset (&si, 0, sizeof si);
   si.cb = sizeof (si);
   si.dwFlags = STARTF_USESTDHANDLES | STARTF_USESHOWWINDOW;
-  si.wShowWindow = DEBUG_W32_SPAWN? SW_SHOW : SW_MINIMIZE;
+  si.wShowWindow = DEBUG_W32_SPAWN? SW_SHOW : SW_HIDE;
   si.hStdInput  = inpipe[0]  == INVALID_HANDLE_VALUE? nullhd[0] : inpipe[0];
   si.hStdOutput = outpipe[1] == INVALID_HANDLE_VALUE? nullhd[1] : outpipe[1];
   si.hStdError  = errpipe[1] == INVALID_HANDLE_VALUE? nullhd[2] : errpipe[1];
@@ -553,20 +566,34 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
               | ((flags & GNUPG_SPAWN_DETACHED)? DETACHED_PROCESS : 0)
               | GetPriorityClass (GetCurrentProcess ())
               | CREATE_SUSPENDED);
-/*   log_debug ("CreateProcess, path='%s' cmdline='%s'\n", pgmname, cmdline); */
-  if (!CreateProcess (pgmname,       /* Program to start.  */
-                      cmdline,       /* Command line arguments.  */
-                      &sec_attr,     /* Process security attributes.  */
-                      &sec_attr,     /* Thread security attributes.  */
-                      TRUE,          /* Inherit handles.  */
-                      cr_flags,      /* Creation flags.  */
-                      NULL,          /* Environment.  */
-                      NULL,          /* Use current drive/directory.  */
-                      &si,           /* Startup information. */
-                      &pi            /* Returns process information.  */
-                      ))
+  /*   log_debug ("CreateProcess, path='%s' cmdline='%s'\n", */
+  /*              pgmname, cmdline); */
+  /* Take care: CreateProcessW may modify wpgmname */
+  if (!(wpgmname = utf8_to_wchar (pgmname)))
+    rc = 0;
+  else if (!(wcmdline = utf8_to_wchar (cmdline)))
+    rc = 0;
+  else
+    rc = CreateProcessW (wpgmname,      /* Program to start.  */
+                         wcmdline,      /* Command line arguments.  */
+                         &sec_attr,     /* Process security attributes.  */
+                         &sec_attr,     /* Thread security attributes.  */
+                         TRUE,          /* Inherit handles.  */
+                         cr_flags,      /* Creation flags.  */
+                         NULL,          /* Environment.  */
+                         NULL,          /* Use current drive/directory.  */
+                         &si,           /* Startup information. */
+                         &pi            /* Returns process information.  */
+                         );
+  if (!rc)
     {
-      log_error ("CreateProcess failed: %s\n", w32_strerror (-1));
+      if (!wpgmname || !wcmdline)
+        log_error ("CreateProcess failed (utf8_to_wchar): %s\n",
+                   strerror (errno));
+      else
+        log_error ("CreateProcess failed: %s\n", w32_strerror (-1));
+      xfree (wpgmname);
+      xfree (wcmdline);
       xfree (cmdline);
       if (infp)
         es_fclose (infp);
@@ -588,6 +615,8 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
         CloseHandle (errpipe[1]);
       return gpg_err_make (errsource, GPG_ERR_GENERAL);
     }
+  xfree (wpgmname);
+  xfree (wcmdline);
   xfree (cmdline);
   cmdline = NULL;
 
@@ -649,9 +678,11 @@ gnupg_spawn_process_fd (const char *pgmname, const char *argv[],
   gpg_error_t err;
   SECURITY_ATTRIBUTES sec_attr;
   PROCESS_INFORMATION pi = { NULL, 0, 0, 0 };
-  STARTUPINFO si;
+  STARTUPINFOW si;
   char *cmdline;
-  int i;
+  wchar_t *wcmdline = NULL;
+  wchar_t *wpgmname = NULL;
+  int i, rc;
   HANDLE stdhd[3];
 
   /* Setup return values.  */
@@ -679,25 +710,38 @@ gnupg_spawn_process_fd (const char *pgmname, const char *argv[],
   si.hStdError  = errfd == -1? stdhd[2] : (void*)_get_osfhandle (errfd);
 
 /*   log_debug ("CreateProcess, path='%s' cmdline='%s'\n", pgmname, cmdline); */
-  if (!CreateProcess (pgmname,       /* Program to start.  */
-                      cmdline,       /* Command line arguments.  */
-                      &sec_attr,     /* Process security attributes.  */
-                      &sec_attr,     /* Thread security attributes.  */
-                      TRUE,          /* Inherit handles.  */
-                      (CREATE_DEFAULT_ERROR_MODE
-                       | GetPriorityClass (GetCurrentProcess ())
-                       | CREATE_SUSPENDED | DETACHED_PROCESS),
-                      NULL,          /* Environment.  */
-                      NULL,          /* Use current drive/directory.  */
-                      &si,           /* Startup information. */
-                      &pi            /* Returns process information.  */
-                      ))
+  /* Take care: CreateProcessW may modify wpgmname */
+  if (!(wpgmname = utf8_to_wchar (pgmname)))
+    rc = 0;
+  else if (!(wcmdline = utf8_to_wchar (cmdline)))
+    rc = 0;
+  else
+    rc = CreateProcessW (wpgmname,      /* Program to start.  */
+                         wcmdline,      /* Command line arguments.  */
+                         &sec_attr,     /* Process security attributes.  */
+                         &sec_attr,     /* Thread security attributes.  */
+                         TRUE,          /* Inherit handles.  */
+                         (CREATE_DEFAULT_ERROR_MODE
+                          | GetPriorityClass (GetCurrentProcess ())
+                          | CREATE_SUSPENDED | DETACHED_PROCESS),
+                         NULL,          /* Environment.  */
+                         NULL,          /* Use current drive/directory.  */
+                         &si,           /* Startup information. */
+                         &pi            /* Returns process information.  */
+                         );
+  if (!rc)
     {
-      log_error ("CreateProcess failed: %s\n", w32_strerror (-1));
+      if (!wpgmname || !wcmdline)
+        log_error ("CreateProcess failed (utf8_to_wchar): %s\n",
+                   strerror (errno));
+      else
+        log_error ("CreateProcess failed: %s\n", w32_strerror (-1));
       err = my_error (GPG_ERR_GENERAL);
     }
   else
     err = 0;
+  xfree (wpgmname);
+  xfree (wcmdline);
   xfree (cmdline);
   for (i=0; i < 3; i++)
     if (stdhd[i] != INVALID_HANDLE_VALUE)
@@ -841,16 +885,20 @@ gnupg_spawn_process_detached (const char *pgmname, const char *argv[],
       0,         /* Returns pid.  */
       0          /* Returns tid.  */
     };
-  STARTUPINFO si;
+  STARTUPINFOW si;
   int cr_flags;
   char *cmdline;
-
+  wchar_t *wcmdline = NULL;
+  wchar_t *wpgmname = NULL;
+  BOOL in_job = FALSE;
+  gpg_err_code_t ec;
+  int rc;
 
   /* We don't use ENVP.  */
   (void)envp;
 
-  if (access (pgmname, X_OK))
-    return my_error_from_syserror ();
+  if ((ec = gnupg_access (pgmname, X_OK)))
+    return gpg_err_make (default_errsource, ec);
 
   /* Prepare security attributes.  */
   memset (&sec_attr, 0, sizeof sec_attr );
@@ -872,24 +920,83 @@ gnupg_spawn_process_detached (const char *pgmname, const char *argv[],
               | GetPriorityClass (GetCurrentProcess ())
               | CREATE_NEW_PROCESS_GROUP
               | DETACHED_PROCESS);
-/*   log_debug ("CreateProcess(detached), path='%s' cmdline='%s'\n", */
-/*              pgmname, cmdline); */
-  if (!CreateProcess (pgmname,       /* Program to start.  */
-                      cmdline,       /* Command line arguments.  */
-                      &sec_attr,     /* Process security attributes.  */
-                      &sec_attr,     /* Thread security attributes.  */
-                      FALSE,         /* Inherit handles.  */
-                      cr_flags,      /* Creation flags.  */
-                      NULL,          /* Environment.  */
-                      NULL,          /* Use current drive/directory.  */
-                      &si,           /* Startup information. */
-                      &pi            /* Returns process information.  */
-                      ))
+
+  /* Check if we were spawned as part of a Job.
+   * In a job we need to add CREATE_BREAKAWAY_FROM_JOB
+   * to the cr_flags, otherwise our child processes
+   * are killed when we terminate. */
+  if (!IsProcessInJob (GetCurrentProcess(), NULL, &in_job))
     {
-      log_error ("CreateProcess(detached) failed: %s\n", w32_strerror (-1));
+      log_error ("IsProcessInJob() failed: %s\n", w32_strerror (-1));
+      in_job = FALSE;
+    }
+
+  if (in_job)
+    {
+      /* Only try to break away from job if it is allowed, otherwise
+       * CreateProcess() would fail with an "Access is denied" error. */
+      JOBOBJECT_EXTENDED_LIMIT_INFORMATION info;
+      if (!QueryInformationJobObject (NULL, JobObjectExtendedLimitInformation,
+                                      &info, sizeof info, NULL))
+        {
+          log_error ("QueryInformationJobObject() failed: %s\n",
+                     w32_strerror (-1));
+        }
+      else if ((info.BasicLimitInformation.LimitFlags &
+                JOB_OBJECT_LIMIT_BREAKAWAY_OK))
+        {
+          log_debug ("Using CREATE_BREAKAWAY_FROM_JOB flag\n");
+          cr_flags |= CREATE_BREAKAWAY_FROM_JOB;
+        }
+      else if ((info.BasicLimitInformation.LimitFlags &
+                JOB_OBJECT_LIMIT_SILENT_BREAKAWAY_OK))
+        {
+          /* The child process should automatically detach from the job. */
+          log_debug ("Not using CREATE_BREAKAWAY_FROM_JOB flag; "
+                     "JOB_OBJECT_LIMIT_SILENT_BREAKAWAY_OK is set\n");
+        }
+      else
+        {
+          /* It seems that the child process must remain in the job.
+           * This is not necessarily an error, although it can cause premature
+           * termination of the child process when the job is closed. */
+          log_debug ("Not using CREATE_BREAKAWAY_FROM_JOB flag\n");
+        }
+    }
+
+  /*   log_debug ("CreateProcess(detached), path='%s' cmdline='%s'\n", */
+  /*              pgmname, cmdline); */
+  /* Take care: CreateProcessW may modify wpgmname */
+  if (!(wpgmname = utf8_to_wchar (pgmname)))
+    rc = 0;
+  else if (!(wcmdline = utf8_to_wchar (cmdline)))
+    rc = 0;
+  else
+    rc = CreateProcessW (wpgmname,      /* Program to start.  */
+                         wcmdline,      /* Command line arguments.  */
+                         &sec_attr,     /* Process security attributes.  */
+                         &sec_attr,     /* Thread security attributes.  */
+                         FALSE,         /* Inherit handles.  */
+                         cr_flags,      /* Creation flags.  */
+                         NULL,          /* Environment.  */
+                         NULL,          /* Use current drive/directory.  */
+                         &si,           /* Startup information. */
+                         &pi            /* Returns process information.  */
+                         );
+  if (!rc)
+    {
+      if (!wpgmname || !wcmdline)
+        log_error ("CreateProcess failed (utf8_to_wchar): %s\n",
+                   strerror (errno));
+      else
+        log_error ("CreateProcess(detached) failed: %s\n", w32_strerror (-1));
+      xfree (wpgmname);
+      xfree (wcmdline);
       xfree (cmdline);
       return my_error (GPG_ERR_GENERAL);
     }
+  xfree (wpgmname);
+  xfree (wcmdline);
   xfree (cmdline);
   cmdline = NULL;
 
